package info.bukova.isspst.services.requirement;

import info.bukova.isspst.data.AuthItem;
import info.bukova.isspst.data.JobMapping;
import info.bukova.isspst.data.RequirementBase;
import info.bukova.isspst.data.RequirementState;
import info.bukova.isspst.data.Role;
import info.bukova.isspst.data.User;
import info.bukova.isspst.data.Workflow;
import info.bukova.isspst.data.Workgroup;
import info.bukova.isspst.services.AbstractOwnedService;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;

public abstract class RequirementBaseServiceImpl<T extends RequirementBase> extends
		AbstractOwnedService<T> implements RequirementBaseService<T> {
	
	protected void addWorkflow(T entity) {
		if (entity.getType() == null) {
			return;
		}
		
		for (Workflow w : entity.getType().getWorkflow()) {
			Workflow workflow = new Workflow();
			workflow.setCentre(w.getCentre());
			workflow.setLimit(w.getLimit());
			workflow.setOrder(w.getOrder());
			workflow.setRole(w.getRole());
			entity.getWorkflow().add(workflow);
		}
	}

	@Override
	@Transactional
	public List<User> getNextApprover(T entity) {
		T e = (T) dao.getById(entity.getId());
		
		
		List<User> users = new ArrayList<User>();
		Workgroup wg;
		Workflow wf = getNextWorkflow(e);
		Role nextRole;
		
		if (wf == null) {
			return null;
		} else {
			nextRole = wf.getRole();
		}
		
		if (wf.getCentre() || e.getWorkgroup() == null) {
			wg = e.getCentre();
		} else {
			wg = e.getWorkgroup();
		}
		
		for (JobMapping jm : wg.getMembers()) {
			if (jm.getRole().equals(nextRole) && jm.getMember().isPerson()) {
				users.add((User) jm.getMember());
			}
		}
		
		return users;
	}

	@Override
	@Transactional
	public void loadAuthItems(T entity) {
		T e = dao.getById(entity.getId());
		e.getAuthorization().size();
		entity.setAuthorization(e.getAuthorization());
	}
	

	@Override
	@Transactional
	public void loadType(T data) {
		T e = dao.getById(data.getId());
		e.getType().getType();
		data.setType(e.getType());
	}

	@Override
	@Transactional
	public void loadWorkflow(T data) {
		T e = dao.getById(data.getId());
		e.getWorkflow().size();
		data.setWorkflow(e.getWorkflow());
	}

	@Override
	@Transactional
	@PreAuthorize("this.canApprove(#entity)")
	public void approve(T entity) {
		T e = (T) dao.getById(entity.getId());
		
		Workflow wf = getNextWorkflow(e);
		if (wf == null) {
			return;
		}
		
		Role role = wf.getRole();
		AuthItem auth = new AuthItem();
		auth.setApprover(getLoggedInUser());
		auth.setRole(role);
		auth.setAuthDate(new Date());
		
		e.getAuthorization().add(auth);
		
		if (getNextWorkflow(e) == null) {
			e.setState(RequirementState.APPROVED);
		} else {
			e.setState(RequirementState.PARTIALLY);
		}
		entity.setState(e.getState());
		entity.getAuthorization().add(auth);
		
		this.update(e);
	}

	@Override
	@Transactional
	public boolean canApprove(T entity) {
		List<User> nextApprovers = getNextApprover(entity); 
		if (nextApprovers != null && nextApprovers.contains(getLoggedInUser())) {
			return true;
		}
		
		return false;
	}
	
	private Workflow getNextWorkflow(T e) {
		AuthItem authItem = null;
		if (e.getWorkflow() == null) {
			return null;
		}
		
		if (e.getAuthorization().size() > 0) {
			authItem = e.getAuthorization().get(e.getAuthorization().size() - 1); 
		}
		
		Workflow wf = null;
		for (int i = 0; i < e.getWorkflow().size(); i++) {
			wf = e.getWorkflow().get(i);
			if (authItem != null && wf.getRole().equals(authItem.getRole())) {
				if (i + 1 < e.getWorkflow().size()) {
					wf = e.getWorkflow().get(i + 1);
					break;
				} else {
					return null;
				}
			}
			
			if (authItem == null) {
				break;
			}
		}
		
		return wf;
	}

}
